import os, sys, unittest
import preppy
from check_basics import fposto, mkSuite

class ImportTestCase(unittest.TestCase):
    def setUp(self):
        self.cwd = os.getcwd()
        dn = os.path.dirname(sys.argv[0])
        if dn: os.chdir(dn)
        preppy.installImporter()

    @fposto
    def testImport1(self):
        if os.path.isfile('sample001.pyc'):
            os.remove('sample001.pyc')
        import sample001
        sample001.getOutput({})

        #uninstallImporter seems not to work

    @fposto
    def testImport2(self):
        if os.path.isfile('sample001n.pyc'):
            os.remove('sample001n.pyc')
        import sample001n
        sample001n.get(A=4)

    @fposto
    def testImport3(self):
        parentDir = os.path.normpath(os.path.join(os.getcwd(),'..'))
        sys.path.insert(0,parentDir)
        try:
            for name in 'sample001 sample001n'.split():
                if name in sys.modules:
                    del sys.modules[name]
            import test.sample001, test.sample001n
        finally:
            sys.path.remove(parentDir)

    def tearDown(self):
        preppy.uninstallImporter()
        if self.cwd!=os.getcwd():
            os.chdir(self.cwd)

class PythonImportTestCase(unittest.TestCase):
    def setUp(self):
        self.cwd = os.getcwd()
        dn = os.path.dirname(sys.argv[0])
        if dn: os.chdir(dn)

    @fposto
    def testPythonImport1(self):
        self.assertTrue(os.path.isfile('sample001.pyc'))
        if 'sample001' in sys.modules:
            del sys.modules['sample001']
        import sample001
        sample001.getOutput({})

    @fposto
    def testPythonImport2(self):
        self.assertTrue(os.path.isfile('sample001n.pyc'))
        if 'sample001n' in sys.modules:
            del sys.modules['sample001n']
        import sample001n
        sample001n.get(A=4)

    def tearDown(self):
        preppy.uninstallImporter()
        if self.cwd!=os.getcwd():
            os.chdir(self.cwd)

def makeSuite():
    suite = mkSuite(ImportTestCase)
    suite.addTests(mkSuite(PythonImportTestCase))
    return suite

if __name__=='__main__':
    runner = unittest.TextTestRunner()
    runner.run(makeSuite())



# Copyright ReportLab Europe Ltd. 2000-2015
# see license.txt for license details

# Basic test suite for Preppy.py
# See check_load.py for load testing

# $Id$

import os, glob, functools
import preppy
import unittest

def fposto(f):
    '''force preppy override stdout'''
    @functools.wraps(f)
    def wrapper(*args,**kwds):
        osto = os.environ.get('PREPPY_OVERRIDE_STDOUT',None)
        os.environ['PREPPY_OVERRIDE_STDOUT'] = '1'
        try:
            return f(*args,**kwds)
        finally:
            if osto is None:
                del os.environ['PREPPY_OVERRIDE_STDOUT']
            else:
                os.environ['PREPPY_OVERRIDE_STDOUT'] = osto
    return wrapper

def mkSuite(cls,prefix=None):
    loader = unittest.TestLoader()
    if prefix: loader.testMethodPrefix = prefix
    return loader.loadTestsFromTestCase(cls)
        
class SimpleTestCase(unittest.TestCase):
    def setUp(self):
        import check_basics
        self.dirName = os.path.dirname(check_basics.__file__)

    def check01ScriptTag(self):
        processTest('sample001')

    def check02ForAndWhileLoops(self):
        processTest('sample002')

    def check03IfTags(self):
        processTest('sample003')

    def check04PassingDictionaries(self):
        dictionary = {"song1": {"songtitle": "The Lumberjack Song", "by": "Monthy Python"},
                      "song2": {"songtitle": "My Way", "by": "Frank Sinatra"}}
        processTest('sample004', dictionary)

    def check05Escapes(self):
        processTest('sample005')

    @fposto
    def check06NoPythonFile(self):
        mod = preppy.getModule(os.path.join(self.dirName, 'sample006'), savefile=0)
        outFile = open(os.path.join(self.dirName, 'sample006.html'), 'w')
        mod.run(dictionary={}, outputfile = outFile)
        outFile.close()

    def check07NoOutputFile(self):
        sourceString = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <HTML>
        <HEAD>
        <TITLE>ReportLab Preppy Test Suite 007</TITLE>
        </HEAD>

        <BODY>
        <FONT COLOR=#000000>
        <TABLE BGCOLOR=#0000CC BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=100%>
        <TR>
        <TD>
        <FONT COLOR=#FFFFFF>
        <CENTER>
        <H1>Preppy Test 007 - Creating A Module Without Using The Filesystem</H1>
        </CENTER>
        </FONT>
        </TD>
        </TR>
        </TABLE>

        <BR>

        This test creates a preppy source module in memory. It does <I>not</I> have a preppy source file (a file ending in '.prep')

        <H2>Expected Output</H2>
        <P>

        Below is a listing of the files in the same directory as this HTML file. There should be no file called <I>sample007.prep</I>.


        <BR><HR>
        <BR><CENTER>
        <TABLE>
        {{script}}
        import os
        leftColumn = []
        rightColumn = []

        fileList = os.listdir(os.getcwd())

        for item in range(0,len(fileList),2):
            if os.path.isfile(fileList[item]):
                leftColumn.append(fileList[item])

        for item in range(1,len(fileList)-1,2):
            if os.path.isfile(fileList[item]):
                rightColumn.append(fileList[item])

        for loopCounter in range(0, len(leftColumn)):
            __write__("<TR><TD WIDTH=30%>")
            __write__(leftColumn[loopCounter])
            __write__("</TD>")
            __write__("<TD WIDTH=30%>")
            try:
                __write__(rightColumn[loopCounter])
                __write__("</TD></TR>")
            except:
                __write__(" ")
                __write__("</TD></TR>")
                {{endscript}}
        </TD></TR>
        </TABLE>
        </CENTER>

        <BR>
        <HR>

        </FONT>
        </BODY>
        </HTML>"""

        mod = preppy.getModule("dummy_module", sourcetext=sourceString)
        modDataList=[]
        tempDict={'temp':'temporary variable'}

        # this way it goes to the string
        #from reportlab.lib.utils import getStringIO
        #f = getStringIO()
        #mod.run(tempDict, outputfile=f)
        #f.seek(0)
        #output = f.read()

        #this way it goes to the list and also gets printed
        mod.run(tempDict, __write__ = modDataList.append)
        #print('length of list is %d' % len(modDataList))
        output=''.join(modDataList)
        outFile=open('sample007.html', 'w')
        outFile.write(output)
        outFile.close()

    def check08_StringKeysThatLookLikeIntegers(self):
        # This test case now works as expected
        processTest('sample008',
                    {'1':'The key for this dictionary item looks like a number, but is actually a string',
                     '2':'So is this one - it looks like an integer'}
                    )

    def check09_StringKeysThatLookLikeFloats(self):
        # This test case now works as expected too
        processTest('sample009',
                    {'1.01':'The key for this dictionary item looks like a number, but is actually a string',
                     '2.78':'So is this one - it looks like a floating point number'}
                    )

    def check10_PreppyInternals_VariableNames(self):
        # This list contains a number of names of variables that are/might be used internally within Preppy.
        preppyVariables = ["STARTDELIMITER", "ENDDELIMITER",
        "QSTARTDELIMITER", "QENDDELIMITER", "QUOTE", "QUOTEQUOTE",
        "UNESCAPES", "VERSION", "KEYWORDS","out","a", "start",
        "lines", "firstpass", "linemax", "result","s", "x",
        "Substringquotes", "cursor", "lineno", "DIAGNOSTIC_FUNCTION",
        "dictlines", "adddict", "done", "starttaglen", "endtaglen",
        "dictassntemplate", "key_seen", "findpercent", "findstarttag",
        "findmin", "sclean", "cursor", "findendtag", "savecursor",
        "ValueError", "block", "blockx", "sblock", "c", "rc",
        "substitutionline", "dictassntemplate", "stringsub",
        "addpart", "teststring", "yislonger", "loop",
        "TOPLEVELWRAPPER", "__save_sys_stdout__", "TOPLEVELPROLOG",
        "outputfile", "NameError", "__write__", "__d__", "__n__",
        "__c__", "__forbidden__", "indentstring", "toplevel",
        "DIAGNOSTIC_FUNCTION", "segment", "outputlist", "prints",
        "newcursor", "oldcursor", "inputtext", "startblock", "sblock",
        "conditional", "iconditional", "inif", "complexdirective",
        "inif", "directive", "ddblock", "blockmode",
        "compilemode","endtag", "endlocation", "inputtext",
        "endtag","t", "v", "tb", "code", "output", "starttaglen",
        "endtaglen", "newcursor", "inputtext", "findendtag",
        "startblock", "endblock", "block", "iout", "result",
        "module_source_template", "lk", "test", "lines",
        "module_source_template" "line0", "sline0", "ssline0",
        "firstword", "findfirstword", "indent", "linesout", "l",
        "lindent", "fn", "P", "out", "name", "result",
        "GLOBAL_LOADED_MODULE_DICTIONARY", "sourcetext", "verbose",
        "sourcefilename", "savefile", "module", "checksum",
        "sourcechecksum", "out", "outfilename", "outfile", "result",
        "textlines", "i", "l", "l1", "l2", "l3","getModule" ]
        dictionary = {}
        for f in range(0, len(preppyVariables)):
            tempvar = preppyVariables[f]
            #print("tempvar = %r" %tempvar)
            dictionary[tempvar] = tempvar
        processTest('sample010', dictionary)

    def check11_PreppyInternals_FunctionNames(self):
        # This lists contains a number of names of functions that are used internally within Preppy.
        preppyFunctions = ["unescape", "printstmt", "countnewlines",
        "quotestring" ,"diag" ,"slen", "quotedparts", "addpart",
        "indent", "__call__", "diag", "check_condition",
        "strip", "getDirectiveBlock", "module_source",
        "proctologist", "check_condition", "complexdirective",
        "dedent", "parsetest", "testgetmodule", "getPreppyModule",
        "cleantext", "new_module", "getPreppyModule"]
        dictionary = {}
        for f in range(0, len(preppyFunctions)):
            tempvar = preppyFunctions[f]
            #print("tempvar = %r" %tempvar)
            dictionary[tempvar] = tempvar
        processTest('sample011', dictionary)

    def check12_quotingOverride(self):
        processTest('sample012')

suite = mkSuite(SimpleTestCase,'check')

@fposto
def processTest(filename, dictionary={}):
    #this ensures we search for prep files in the module
    #where this tests lives, wherever we invoked them from.
    import check_basics
    dirName = os.path.dirname(check_basics.__file__)
    #print('processTest: %r'%dictionary)
    root, ext = os.path.splitext(filename)
    outFileName = os.path.join(dirName, root + '.html')
    mod = preppy.getModule(os.path.join(dirName, root))
    outFile = open(outFileName, 'w')
    mod.run(dictionary, outputfile = outFile)
    outFile.close()
    #print('wrote %r' %outFileName)


def clean(dirname='.'):
    for filename in glob.glob('sample*.prep'):
        root, ext = os.path.splitext(filename)
        if os.path.isfile(root + '.html'):
            os.remove(root + '.html')
        if os.path.isfile(root + '.py'):
            os.remove(root + '.py')
        if os.path.isfile(root + '.pyc'):
            os.remove(root + '.pyc')
    for filename in glob.glob('*TestFile*.txt'):
        os.remove(filename)
    # Get rid of dynamically generated stuff not caught by the above code...
    try:
        os.remove('sample007.html')
        os.remove('sample013.prep')
        os.remove('sample014.prep')
    except:
        pass

if __name__=='__main__':
    import sys
    if len(sys.argv) == 2:
        arg = sys.argv[1]
        if arg == 'clean':
            clean()
            sys.exit()

        else:
            print('argument not recognised!')
    else:
        runner = unittest.TextTestRunner()
        runner.run(suite)
        print('\nplease read all sample*.html files')



# Copyright ReportLab Europe Ltd. 2000-2015
# see license.txt for license details
# runs all key preppy tests.  Assumes preppy
# is on the path.
import os, sys
import unittest
from check_basics import mkSuite

def makeSuite():
    import check_basics, check_algorithms, check_load, check_lowlevel, test_import_hook
    suite = check_algorithms.makeSuite()
    suite.addTests(check_basics.suite)
    suite.addTests(mkSuite(check_load.LoadTestCase,'load'))
    suite.addTests(check_lowlevel.makeSuite())
    suite.addTests(test_import_hook.makeSuite())
    return suite

if __name__=='__main__':

    #try to pick up the preppy.py in the directory above
    parentDir = os.path.dirname(os.getcwd())
    sys.path.insert(0, parentDir)

    runner = unittest.TextTestRunner()
    result = runner.run(makeSuite())
    import sys
    sys.exit(not result.wasSuccessful)



# Copyright ReportLab Europe Ltd. 2000-2015
# see license.txt for license details

# Tests of various functions and algorithms in preppy.
# no side-effects on file system, run anywhere.
__version__=''' $Id$ '''
import sys, os, glob, string, re
import preppy
import unittest
from check_basics import mkSuite
isPy3 = preppy.isPy3
if isPy3:
    xrange = range
isPy34 = preppy.isPy34
isPy35 = isPy3 and sys.version_info[1]>=5
isPy38 = preppy.isPy38
isPy39 = preppy.isPy39
isPy310 = preppy.isPy310
isPy311 = preppy.isPy311
isPy313 = preppy.isPy313

def P__tokenize(source,filename='<unknown>'):
    P=preppy.PreppyParser(source,filename)
    return P._PreppyParser__tokenize()

def normValue(s, pat=re.compile(r'([\x28\x5b])\s+',re.M)):
    return pat.sub(r'\1',s)

def P__preppy(source,filename='<unknown>', expected=None, params={}):
    if  expected is not None:
        mod=preppy.getModule(filename,savePyc=0,sourcetext=source)
        collector = []
        mod.run(params, __write__=collector.append, quoteFunc=str, lquoteFunc=None)
        output = ''.join(collector)
        if output!=expected:
            raise ValueError('P__preppy filename=%s:  expected %r\ngot %r'%(filename,expected,output))

    P=preppy.PreppyParser(source,filename)
    P._PreppyParser__tokenize()
    return P.dump(P._PreppyParser__preppy())

class PreppyParserTestCase(unittest.TestCase):
    '''test the preppy parser class'''

    def __init__(self,*args,**kwds):
        unittest.TestCase.__init__(self,*args,**kwds)
        self.maxDiff = None

    @staticmethod
    def fixup(s):
        sNN = ' None, None,'
        if isPy3:
            sTE = 'Try'
            sTFL = ' [],'
            if isPy35: sNN = ''
            sEL = ', []'
            if isPy313: sEL=''
        else:
            sTE = 'TryExcept'
            sTFL = ''
        return s.replace('sTE',sTE).replace('sTFL',sTFL).replace('sNN',sNN).replace('sEL',sEL)

    def assertEqualStr(self,a,b):
        a = normValue(a)
        b = normValue(self.fixup(b))
        for i in xrange(max(len(a),len(b))):
            if a[i:i+1]!=b[i:i+1]:
                raise ValueError('Not Equal\n%r !!!!! %r\n%r !!!!! %r' % (a[:i],a[i:],b[:i],b[i:]))

    def checkTokenize(self):
        Token = preppy.Token
        self.assertEqual(P__tokenize('Hello World!'),[('const', 0, 12), ('eof', 12, 12)])
        self.assertEqual(P__tokenize('{{1}}'),[('expr', 2, 3), ('eof', 5, 5)])
        self.assertEqual(P__tokenize('{{while 1}}'),[Token(kind='while', start=2, end=9), Token(kind='eof', start=11, end=11)])
        self.assertEqual(P__tokenize('{{for i in 1,2}}'),[Token(kind='for', start=2, end=14), Token(kind='eof', start=16, end=16)])
        self.assertEqual(P__tokenize('{{if 1}}'), [Token(kind='if', start=2, end=6), Token(kind='eof', start=8, end=8)])
        self.assertEqual(P__tokenize('{{script}}a=2{{endscript}}'),[('script', 2, 8), ('const', 10, 13), ('endscript', 15, 24), ('eof', 26, 26)])
        self.assertEqual(P__tokenize('{{script}}a=1\nb=2\nc=3{{endscript}}'), [Token(kind='script', start=2, end=8), Token(kind='const', start=10, end=21), Token(kind='endscript', start=23, end=32), Token(kind='eof', start=34, end=34)])

    if isPy38:
        results=dict(
                check_break = "[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)]sEL, lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), If(Compare(Name('i', Load(), lineno=1, col_offset=29, end_lineno=1, end_col_offset=37), [Eq()], [Constant(3, lineno=1, col_offset=32, end_lineno=1, end_col_offset=40)], lineno=1, col_offset=29, end_lineno=1, end_col_offset=40), [Break(lineno=1, col_offset=37, end_lineno=1, end_col_offset=42)]sEL, lineno=1, col_offset=26, end_lineno=1, end_col_offset=51)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=61, end_lineno=1, end_col_offset=66), [Constant('eeeee', lineno=1, col_offset=61, end_lineno=1, end_col_offset=66)]sEL, lineno=1, col_offset=61, end_lineno=1, end_col_offset=66), lineno=1, col_offset=61, end_lineno=1, end_col_offset=66)], lineno=1, col_offset=2, end_lineno=1, end_col_offset=74)]",
                check_continue = "[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)]sEL, lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), If(Compare(Name('i', Load(), lineno=1, col_offset=29, end_lineno=1, end_col_offset=37), [Eq()], [Constant(3, lineno=1, col_offset=32, end_lineno=1, end_col_offset=40)], lineno=1, col_offset=29, end_lineno=1, end_col_offset=40), [Continue(lineno=1, col_offset=37, end_lineno=1, end_col_offset=45)]sEL, lineno=1, col_offset=26, end_lineno=1, end_col_offset=54)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=64, end_lineno=1, end_col_offset=69), [Constant('eeeee', lineno=1, col_offset=64, end_lineno=1, end_col_offset=69)]sEL, lineno=1, col_offset=64, end_lineno=1, end_col_offset=69), lineno=1, col_offset=64, end_lineno=1, end_col_offset=69)], lineno=1, col_offset=2, end_lineno=1, end_col_offset=77)]",
                check_eval = "[Expr(Call(Name('__swrite__', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=0), [Expr(Call(Attribute(Name('str', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=3), 'split', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=9), [Constant('this should show a list of strings', lineno=2, col_offset=0, end_lineno=4, end_col_offset=36)]sEL, lineno=1, col_offset=8, end_lineno=4, end_col_offset=37), lineno=1, col_offset=8, end_lineno=4, end_col_offset=37)]sEL, lineno=1, col_offset=8, end_lineno=3, end_col_offset=0), lineno=1, col_offset=8, end_lineno=3, end_col_offset=0)]",
                check_for = "[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)]sEL, lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)]sEL, lineno=1, col_offset=2, end_lineno=1, end_col_offset=32)]",
                check_while = "[While(Compare(Name('i', Load(), lineno=1, col_offset=8, end_lineno=1, end_col_offset=19), [Lt()], [Constant(10, lineno=1, col_offset=10, end_lineno=1, end_col_offset=22)], lineno=1, col_offset=8, end_lineno=1, end_col_offset=22), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=14, end_lineno=1, end_col_offset=20), [Constant('abcdef', lineno=1, col_offset=14, end_lineno=1, end_col_offset=20)]sEL, lineno=1, col_offset=14, end_lineno=1, end_col_offset=20), lineno=1, col_offset=14, end_lineno=1, end_col_offset=20)]sEL, lineno=1, col_offset=2, end_lineno=1, end_col_offset=30)]",
                check_try_0 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)]sEL, lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)]sEL, lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=23, end_lineno=1, end_col_offset=26), [Constant('ccc', lineno=1, col_offset=23, end_lineno=1, end_col_offset=26)]sEL, lineno=1, col_offset=23, end_lineno=1, end_col_offset=26), lineno=1, col_offset=23, end_lineno=1, end_col_offset=26)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=26)]sELsEL, lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_1 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)]sEL, lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)]sEL, lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)]sEL, lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)]sEL, lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)]sELsEL, lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_2 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)]sEL, lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)]sEL, lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)]sEL, lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)]sEL, lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), [Constant('ddd', lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)]sEL, lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)]sEL, lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_3 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)]sEL, lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)]sEL, lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)]sEL, lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(body=[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)]sEL, lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), [Constant('ddd', lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)]sEL, lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=72, end_lineno=1, end_col_offset=75), [Constant('fff', lineno=1, col_offset=72, end_lineno=1, end_col_offset=75)]sEL, lineno=1, col_offset=72, end_lineno=1, end_col_offset=75), lineno=1, col_offset=72, end_lineno=1, end_col_offset=75)], lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_script = "[Assign([Name('a', Store(), lineno=1, col_offset=10, end_lineno=3, end_col_offset=4)], Constant(1, lineno=1, col_offset=12, end_lineno=3, end_col_offset=6), lineno=1, col_offset=10, end_lineno=3, end_col_offset=6), Assign([Name('b', Store(), lineno=2, col_offset=0, end_lineno=4, end_col_offset=4)], Constant(2, lineno=2, col_offset=2, end_lineno=4, end_col_offset=6), lineno=2, col_offset=0, end_lineno=4, end_col_offset=6), Assign([Name('c', Store(), lineno=3, col_offset=0, end_lineno=5, end_col_offset=4)], Constant(3, lineno=3, col_offset=2, end_lineno=5, end_col_offset=6), lineno=3, col_offset=0, end_lineno=5, end_col_offset=6)]",
                ) if isPy39 else dict(
                check_break="[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, None, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, None, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, None, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)], [], lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), If(Compare(Name('i', Load(), lineno=1, col_offset=29, end_lineno=1, end_col_offset=37), [Eq()], [Constant(3, None, lineno=1, col_offset=32, end_lineno=1, end_col_offset=40)], lineno=1, col_offset=29, end_lineno=1, end_col_offset=40), [Break(lineno=1, col_offset=37, end_lineno=1, end_col_offset=42)], [], lineno=1, col_offset=26, end_lineno=1, end_col_offset=51)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=61, end_lineno=1, end_col_offset=66), [Constant('eeeee', lineno=1, col_offset=61, end_lineno=1, end_col_offset=66)], [], lineno=1, col_offset=61, end_lineno=1, end_col_offset=66), lineno=1, col_offset=61, end_lineno=1, end_col_offset=66)], None, lineno=1, col_offset=2, end_lineno=1, end_col_offset=74)]",
                check_continue="[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, None, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, None, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, None, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)], [], lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), If(Compare(Name('i', Load(), lineno=1, col_offset=29, end_lineno=1, end_col_offset=37), [Eq()], [Constant(3, None, lineno=1, col_offset=32, end_lineno=1, end_col_offset=40)], lineno=1, col_offset=29, end_lineno=1, end_col_offset=40), [Continue(lineno=1, col_offset=37, end_lineno=1, end_col_offset=45)], [], lineno=1, col_offset=26, end_lineno=1, end_col_offset=54)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=64, end_lineno=1, end_col_offset=69), [Constant('eeeee', lineno=1, col_offset=64, end_lineno=1, end_col_offset=69)], [], lineno=1, col_offset=64, end_lineno=1, end_col_offset=69), lineno=1, col_offset=64, end_lineno=1, end_col_offset=69)], None, lineno=1, col_offset=2, end_lineno=1, end_col_offset=77)]",
                check_eval="[Expr(Call(Name('__swrite__', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=0), [Expr(Call(Attribute(Name('str', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=3), 'split', Load(), lineno=1, col_offset=8, end_lineno=3, end_col_offset=9), [Constant('this should show a list of strings', None, lineno=2, col_offset=0, end_lineno=4, end_col_offset=36)], [], lineno=1, col_offset=8, end_lineno=4, end_col_offset=37), lineno=1, col_offset=8, end_lineno=4, end_col_offset=37)], [], lineno=1, col_offset=8, end_lineno=3, end_col_offset=0), lineno=1, col_offset=8, end_lineno=3, end_col_offset=0)]",
                check_for="[For(Name('i', Store(), lineno=1, col_offset=6, end_lineno=1, end_col_offset=21), Tuple([Constant(1, None, lineno=1, col_offset=11, end_lineno=1, end_col_offset=26), Constant(2, None, lineno=1, col_offset=13, end_lineno=1, end_col_offset=28), Constant(3, None, lineno=1, col_offset=15, end_lineno=1, end_col_offset=30)], Load(), lineno=1, col_offset=11, end_lineno=1, end_col_offset=30), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), [Constant('abcdef', lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)], [], lineno=1, col_offset=18, end_lineno=1, end_col_offset=24), lineno=1, col_offset=18, end_lineno=1, end_col_offset=24)], [], None, lineno=1, col_offset=2, end_lineno=1, end_col_offset=32)]",
                check_while="[While(Compare(Name('i', Load(), lineno=1, col_offset=8, end_lineno=1, end_col_offset=19), [Lt()], [Constant(10, None, lineno=1, col_offset=10, end_lineno=1, end_col_offset=22)], lineno=1, col_offset=8, end_lineno=1, end_col_offset=22), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=14, end_lineno=1, end_col_offset=20), [Constant('abcdef', lineno=1, col_offset=14, end_lineno=1, end_col_offset=20)], [], lineno=1, col_offset=14, end_lineno=1, end_col_offset=20), lineno=1, col_offset=14, end_lineno=1, end_col_offset=20)], [], lineno=1, col_offset=2, end_lineno=1, end_col_offset=30)]",
                check_try_0="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)], [], lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [], lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=23, end_lineno=1, end_col_offset=26), [Constant('ccc', lineno=1, col_offset=23, end_lineno=1, end_col_offset=26)], [], lineno=1, col_offset=23, end_lineno=1, end_col_offset=26), lineno=1, col_offset=23, end_lineno=1, end_col_offset=26)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=26)], [], [], lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_1="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)], [], lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [], lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], [], lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], [], lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)], [], [], lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_2="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)], [], lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [], lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], [], lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], [], lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), [Constant('ddd', lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)], [], lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)], [], lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_try_3="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), [Constant('aaa', lineno=1, col_offset=0, end_lineno=1, end_col_offset=3)], [], lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), lineno=1, col_offset=0, end_lineno=1, end_col_offset=3), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), [Constant('bbb', lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [], lineno=1, col_offset=10, end_lineno=1, end_col_offset=13), lineno=1, col_offset=10, end_lineno=1, end_col_offset=13)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7, end_lineno=3, end_col_offset=49), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('eee', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], [], lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], lineno=1, col_offset=15, end_lineno=4, end_col_offset=37), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), [Constant('ccc', lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], [], lineno=1, col_offset=47, end_lineno=1, end_col_offset=50), lineno=1, col_offset=47, end_lineno=1, end_col_offset=50)], lineno=1, col_offset=39, end_lineno=4, end_col_offset=50)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), [Constant('ddd', lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)], [], lineno=1, col_offset=58, end_lineno=1, end_col_offset=61), lineno=1, col_offset=58, end_lineno=1, end_col_offset=61)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=72, end_lineno=1, end_col_offset=75), [Constant('fff', lineno=1, col_offset=72, end_lineno=1, end_col_offset=75)], [], lineno=1, col_offset=72, end_lineno=1, end_col_offset=75), lineno=1, col_offset=72, end_lineno=1, end_col_offset=75)], lineno=1, col_offset=5, end_lineno=1, end_col_offset=8)]",
                check_script="[Assign([Name('a', Store(), lineno=1, col_offset=10, end_lineno=3, end_col_offset=4)], Constant(1, None, lineno=1, col_offset=12, end_lineno=3, end_col_offset=6), None, lineno=1, col_offset=10, end_lineno=3, end_col_offset=6), Assign([Name('b', Store(), lineno=2, col_offset=0, end_lineno=4, end_col_offset=4)], Constant(2, None, lineno=2, col_offset=2, end_lineno=4, end_col_offset=6), None, lineno=2, col_offset=0, end_lineno=4, end_col_offset=6), Assign([Name('c', Store(), lineno=3, col_offset=0, end_lineno=5, end_col_offset=4)], Constant(3, None, lineno=3, col_offset=2, end_lineno=5, end_col_offset=6), None, lineno=3, col_offset=0, end_lineno=5, end_col_offset=6)]",
                )
        results['check_if'] = "[If(Name('i', Load(), lineno=1, col_offset=5, end_lineno=1, end_col_offset=10), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=8, end_lineno=1, end_col_offset=11), [Constant('aaa', lineno=1, col_offset=8, end_lineno=1, end_col_offset=11)]sEL, lineno=1, col_offset=8, end_lineno=1, end_col_offset=11), lineno=1, col_offset=8, end_lineno=1, end_col_offset=11)], [If(Name('j', Load(), lineno=1, col_offset=18, end_lineno=1, end_col_offset=25), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=21, end_lineno=1, end_col_offset=24), [Constant('bbb', lineno=1, col_offset=21, end_lineno=1, end_col_offset=24)]sEL, lineno=1, col_offset=21, end_lineno=1, end_col_offset=24), lineno=1, col_offset=21, end_lineno=1, end_col_offset=24)], [If(Name('k', Load(), lineno=1, col_offset=31, end_lineno=1, end_col_offset=38), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), [Constant('ccc', lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)]sEL, lineno=1, col_offset=34, end_lineno=1, end_col_offset=37), lineno=1, col_offset=34, end_lineno=1, end_col_offset=37)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=45, end_lineno=1, end_col_offset=48), [Constant('ddd', lineno=1, col_offset=45, end_lineno=1, end_col_offset=48)]sEL, lineno=1, col_offset=45, end_lineno=1, end_col_offset=48), lineno=1, col_offset=45, end_lineno=1, end_col_offset=48)], lineno=1, col_offset=26, end_lineno=1, end_col_offset=44)], lineno=1, col_offset=13, end_lineno=1, end_col_offset=31)], lineno=1, col_offset=2, end_lineno=1, end_col_offset=55)]"
    else:
        if isPy3:
            _ct3 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0), [Str('aaa', lineno=1, col_offset=0)], [],sNN lineno=1, col_offset=0), lineno=1, col_offset=0), Try([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10), [Str('bbb', lineno=1, col_offset=10)], [],sNN lineno=1, col_offset=10), lineno=1, col_offset=10)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34), [Str('eee', lineno=1, col_offset=34)], [],sNN lineno=1, col_offset=34), lineno=1, col_offset=34)], lineno=1, col_offset=15), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47), [Str('ccc', lineno=1, col_offset=47)], [],sNN lineno=1, col_offset=47), lineno=1, col_offset=47)], lineno=1, col_offset=39)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58), [Str('ddd', lineno=1, col_offset=58)], [],sNN lineno=1, col_offset=58), lineno=1, col_offset=58)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=72), [Str('fff', lineno=1, col_offset=72)], [],sNN lineno=1, col_offset=72), lineno=1, col_offset=72)], lineno=1, col_offset=5)]" % locals()
        else:
            _ct3 = "[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0), [Str('aaa', lineno=1, col_offset=0)], [],sNN lineno=1, col_offset=0), lineno=1, col_offset=0), TryFinally([TryExcept([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10), [Str('bbb', lineno=1, col_offset=10)], [],sNN lineno=1, col_offset=10), lineno=1, col_offset=10)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34), [Str('eee', lineno=1, col_offset=34)], [],sNN lineno=1, col_offset=34), lineno=1, col_offset=34)], lineno=1, col_offset=15), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47), [Str('ccc', lineno=1, col_offset=47)], [],sNN lineno=1, col_offset=47), lineno=1, col_offset=47)], lineno=1, col_offset=39)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58), [Str('ddd', lineno=1, col_offset=58)], [],sNN lineno=1, col_offset=58), lineno=1, col_offset=58)], lineno=1, col_offset=5)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=72), [Str('fff', lineno=1, col_offset=72)], [],sNN lineno=1, col_offset=72), lineno=1, col_offset=72)], lineno=1, col_offset=5)]"
        results=dict(
                check_break="[For(Name('i', Store(), lineno=1, col_offset=6), Tuple([Num(1, lineno=1, col_offset=11), Num(2, lineno=1, col_offset=13), Num(3, lineno=1, col_offset=15)], Load(), lineno=1, col_offset=11), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18), [Str('abcdef', lineno=1, col_offset=18)]sEL,sNN lineno=1, col_offset=18), lineno=1, col_offset=18), If(Compare(Name('i', Load(), lineno=1, col_offset=29), [Eq()], [Num(3, lineno=1, col_offset=32)], lineno=1, col_offset=29), [Break( lineno=1, col_offset=37)], [], lineno=1, col_offset=26)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=61), [Str('eeeee', lineno=1, col_offset=61)]sEL,sNN lineno=1, col_offset=61), lineno=1, col_offset=61)], lineno=1, col_offset=2)]",
                check_continue="[For(Name('i', Store(), lineno=1, col_offset=6), Tuple([Num(1, lineno=1, col_offset=11), Num(2, lineno=1, col_offset=13), Num(3, lineno=1, col_offset=15)], Load(), lineno=1, col_offset=11), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18), [Str('abcdef', lineno=1, col_offset=18)], [],sNN lineno=1, col_offset=18), lineno=1, col_offset=18), If(Compare(Name('i', Load(), lineno=1, col_offset=29), [Eq()], [Num(3, lineno=1, col_offset=32)], lineno=1, col_offset=29), [Continue( lineno=1, col_offset=37)], [], lineno=1, col_offset=26)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=64), [Str('eeeee', lineno=1, col_offset=64)], [],sNN lineno=1, col_offset=64), lineno=1, col_offset=64)], lineno=1, col_offset=2)]",
                check_eval="[Expr(Call(Name('__swrite__', Load(), lineno=1, col_offset=8), [Expr(Call(Attribute(Name('str', Load(), lineno=1, col_offset=8), 'split', Load(), lineno=1, col_offset=%(CO0)s), [Str('this should show a list of strings', lineno=2, col_offset=%(CO1)s)], [],sNN lineno=1, col_offset=%(CO2)s), lineno=1, col_offset=8)], [],sNN lineno=1, col_offset=8), lineno=1, col_offset=8)]",
                check_for="[For(Name('i', Store(), lineno=1, col_offset=6), Tuple([Num(1, lineno=1, col_offset=11), Num(2, lineno=1, col_offset=13), Num(3, lineno=1, col_offset=15)], Load(), lineno=1, col_offset=11), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=18), [Str('abcdef', lineno=1, col_offset=18)], [],sNN lineno=1, col_offset=18), lineno=1, col_offset=18)], [], lineno=1, col_offset=2)]",
                check_if="[If(Name('i', Load(), lineno=1, col_offset=5), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=8), [Str('aaa', lineno=1, col_offset=8)], [],sNN lineno=1, col_offset=8), lineno=1, col_offset=8)], [If(Name('j', Load(), lineno=1, col_offset=18), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=21), [Str('bbb', lineno=1, col_offset=21)], [],sNN lineno=1, col_offset=21), lineno=1, col_offset=21)], [If(Name('k', Load(), lineno=1, col_offset=31), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34), [Str('ccc', lineno=1, col_offset=34)], [],sNN lineno=1, col_offset=34), lineno=1, col_offset=34)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=45), [Str('ddd', lineno=1, col_offset=45)], [],sNN lineno=1, col_offset=45), lineno=1, col_offset=45)], lineno=1, col_offset=26)], lineno=1, col_offset=13)], lineno=1, col_offset=2)]",
                check_while="[While(Compare(Name('i', Load(), lineno=1, col_offset=8), [Lt()], [Num(10, lineno=1, col_offset=10)], lineno=1, col_offset=8), [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=14), [Str('abcdef', lineno=1, col_offset=14)], [],sNN lineno=1, col_offset=14), lineno=1, col_offset=14)], [], lineno=1, col_offset=2)]",
                check_try_0="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0), [Str('aaa', lineno=1, col_offset=0)], [],sNN lineno=1, col_offset=0), lineno=1, col_offset=0), sTE([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10), [Str('bbb', lineno=1, col_offset=10)], [],sNN lineno=1, col_offset=10), lineno=1, col_offset=10)], [ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=23), [Str('ccc', lineno=1, col_offset=23)], [],sNN lineno=1, col_offset=23), lineno=1, col_offset=23)], lineno=1, col_offset=15)], [],sTFL lineno=1, col_offset=5)]",
                check_try_1="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0), [Str('aaa', lineno=1, col_offset=0)], [],sNN lineno=1, col_offset=0), lineno=1, col_offset=0), sTE([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10), [Str('bbb', lineno=1, col_offset=10)], [],sNN lineno=1, col_offset=10), lineno=1, col_offset=10)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34), [Str('eee', lineno=1, col_offset=34)], [],sNN lineno=1, col_offset=34), lineno=1, col_offset=34)], lineno=1, col_offset=15), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47), [Str('ccc', lineno=1, col_offset=47)], [],sNN lineno=1, col_offset=47), lineno=1, col_offset=47)], lineno=1, col_offset=39)], [],sTFL lineno=1, col_offset=5)]",
                check_try_2="[Expr(Call(Name('__write__', Load(), lineno=1, col_offset=0), [Str('aaa', lineno=1, col_offset=0)], [],sNN lineno=1, col_offset=0), lineno=1, col_offset=0), sTE([Expr(Call(Name('__write__', Load(), lineno=1, col_offset=10), [Str('bbb', lineno=1, col_offset=10)], [],sNN lineno=1, col_offset=10), lineno=1, col_offset=10)], [ExceptHandler(Name('ValueError', Load(), lineno=3, col_offset=7), None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=34), [Str('eee', lineno=1, col_offset=34)], [],sNN lineno=1, col_offset=34), lineno=1, col_offset=34)], lineno=1, col_offset=15), ExceptHandler(None, None, [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=47), [Str('ccc', lineno=1, col_offset=47)], [],sNN lineno=1, col_offset=47), lineno=1, col_offset=47)], lineno=1, col_offset=39)], [Expr(Call(Name('__write__', Load(), lineno=1, col_offset=58), [Str('ddd', lineno=1, col_offset=58)], [],sNN lineno=1, col_offset=58), lineno=1, col_offset=58)],sTFL lineno=1, col_offset=5)]",
                check_try_3=_ct3,
                check_script="[Assign([Name('a', Store(), lineno=1, col_offset=10)], Num(1, lineno=1, col_offset=12), lineno=1, col_offset=10), Assign([Name('b', Store(), lineno=2, col_offset=0)], Num(2, lineno=2, col_offset=2), lineno=2, col_offset=0), Assign([Name('c', Store(), lineno=3, col_offset=0)], Num(3, lineno=3, col_offset=2), lineno=3, col_offset=0)]",
                )
        del _ct3

    def check_for(self):
        self.assertEqualStr(P__preppy('{{for i in 1,2,3}}abcdef{{endfor}}',filename='check_for',expected='abcdefabcdefabcdef'), self.results['check_for'])

    def check_break(self):
        self.assertEqualStr(P__preppy('{{for i in 1,2,3}}abcdef{{if i==3}}{{break}}{{endif}}{{else}}eeeee{{endfor}}',filename='check_break',expected='abcdefabcdefabcdef'), self.results['check_break'])

    def check_break_outside_loop_fails(self):
        self.assertRaises(SyntaxError,P__preppy,'{{if i==3}}{{break}}{{endif}}')

    def check_continue(self):
        self.assertEqualStr(P__preppy('{{for i in 1,2,3}}abcdef{{if i==3}}{{continue}}{{endif}}{{else}}eeeee{{endfor}}',filename='check_continue',expected='abcdefabcdefabcdefeeeee'), self.results['check_continue'])

    def check_continue_outside_loop_fails(self):
        self.assertRaises(SyntaxError,P__preppy,'{{if i==3}}{{continue}}{{endif}}')

    def check_while(self):
        self.assertEqualStr(P__preppy('{{while i<10}}abcdef{{endwhile}}',filename='check_while',expected='',params=dict(i=10)), self.results['check_while'])

    def check_eval(self):
        #3.4 seems to improve on the col_offsets
        if isPy34 and sys.version_info[2]<3:
            CO0 = 12
            CO1 = 0
            CO2 = 17
            if isPy35:
                NN = ''
                CO2 = 8
                CO0 = 8
        else:
            CO0 = 8
            CO1 = 0
            CO2 = 8
        self.assertEqualStr(P__preppy('{{eval}}str.split(\n"this should show a list of strings")\n{{endeval}}', filename='check_eval',expected=None), self.results['check_eval'] % vars())

    def check_script(self):
        self.assertEqualStr(P__preppy('{{script}}a=1\nb=2\nc=3{{endscript}}', filename='check_script', expected=''), self.results['check_script'])

    def check_if(self):
        self.assertEqualStr(P__preppy('{{if i}}aaa{{elif j}}bbb{{elif k}}ccc{{else}}ddd{{endif}}', filename='check_if', expected='aaa',params=dict(i=1,k=3)), self.results['check_if'])

    def check_try_0(self):
        if isPy35:
            NN = ''
        self.assertEqualStr(P__preppy('aaa{{try}}bbb{{except}}ccc{{endtry}}', filename='check_try_0', expected='aaabbb'), self.results['check_try_0']%locals())

    def check_try_1(self):
        if isPy35:
            NN = ''
        self.assertEqualStr(P__preppy('aaa{{try}}bbb{{except ValueError}}eee{{except}}ccc{{endtry}}', filename='check_try_1', expected='aaabbb'), self.results['check_try_1']%locals())

    def check_try_2(self):
        self.assertEqualStr(P__preppy('aaa{{try}}bbb{{except ValueError}}eee{{except}}ccc{{else}}ddd{{endtry}}', filename='check_try_2', expected='aaabbbddd'), self.results['check_try_2'] % locals())

    def check_try_3(self):
        self.assertEqualStr(P__preppy('aaa{{try}}bbb{{except ValueError}}eee{{except}}ccc{{else}}ddd{{finally}}fff{{endtry}}', filename='check_try_3', expected='aaabbbdddfff'),self.results['check_try_3'])

    def check_try_4(self):
        self.assertRaises(SyntaxError,P__preppy,'{{try}}bbb{{endtry}}')

    def check_try_5(self):
        self.assertRaises(SyntaxError,P__preppy,'{{try}}bbb{{else}}ddd{{endtry}}')

def makeSuite():
    return mkSuite(PreppyParserTestCase,'check')

if __name__=='__main__':
    runner = unittest.TextTestRunner()
    runner.run(makeSuite())



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import os, sys

if __name__=='__main__':
    pkgDir=os.path.dirname(sys.argv[0])
    if not pkgDir:
        pkgDir=os.getcwd()
    if not os.path.isabs(pkgDir):
        pkgDir=os.path.abspath(pkgDir)
    sys.path.insert(0,pkgDir)
    os.chdir(pkgDir)
    if len(sys.argv)>=2 and sys.argv[1]=='test':
        import subprocess
        def spCall(cmd,*args,**kwds):
            r = subprocess.call(
                    cmd,
                    stderr = subprocess.STDOUT,
                    stdout = subprocess.DEVNULL if kwds.pop('dropOutput',False) else None,
                    timeout = kwds.pop('timeout',3600),
                    )
            if False:
                pfx = '!!!!!' if r else '#####'
                scmd = ' '.join(cmd)
                print(f'{pfx} {scmd} --> {r}')
            return r
        def specialOption(n,ceq=False):
            v = 0
            while n in sys.argv:
                v += 1
                sys.argv.remove(n)
            if ceq:
                n += '='
                V = [_ for _ in sys.argv if _.startswith(n)]
                for _ in V: sys.argv.remove(_)
                if V:
                    n = len(n)
                    v = V[-1][n:]
            return v
        failfast = specialOption('--failfast')
        verboseTests = specialOption('--verbose-tests')
        excludes = [_ for _ in sys.argv if _.startswith('--exclude=')]
        for _ in excludes:
            sys.argv.remove(_)
        if len(sys.argv)!=2:
            raise ValueError('test may only be used alone sys.argv[1:]=%s' % repr(sys.argv[1:]))
        cmd = sys.argv[-1]
        os.chdir(os.path.join(pkgDir,'test'))
        cli = [sys.executable, 'testall.py']+excludes
        if verboseTests:
            cli.append('--verbosity=2')
        if failfast:
            cli.append('--failfast')
        r = spCall(cli)
        sys.exit(f'!!!!! testall.py --> {r} !!!!!' if r else r)

    import preppy
    version = preppy.VERSION

    setup(name='preppy',
        version=version,
        description='preppy - a Preprocessor for Python',
        author='Robin Becker, Andy Robinson, Aaron Watters',
        author_email='andy@reportlab.com',
        url='https://hg.reportlab.com/hg-public/preppy',
        py_modules=['preppy'],
        entry_points = dict(
                        console_scripts = [
                                'preppy=preppy:main',
                                ],
                            ),
        )



# Copyright ReportLab Europe Ltd. 2000-2015
# see license.txt for license details

# Tests of various functions and algorithms in preppy.
# no side-effects on file system, run anywhere.
__version__=''' $Id$ '''
import sys, os, glob, random, unittest, traceback
import preppy
isPy310 = preppy.isPy310
from check_basics import fposto, mkSuite

def checkErrorTextContains(texts,func,*args,**kwds):
    label = kwds.pop('label','')
    if label: label = 'case %s\n' % label
    if not isinstance(texts,(list,tuple)):
        texts = (texts,)
    try:
        func(*args,**kwds)
    except:
        buf=preppy.StringIO()
        traceback.print_exc(file=buf)
        buf = buf.getvalue()
        for t in texts:
            if t not in buf:
                return buf if not label else '%s%s'%(label,buf)
        else:
            return ''
    else:
        return '%serror containing texts\n%s\nwas not raised' % (label,'\n'.join(texts))


class PreppyTestCase(unittest.TestCase):
    @staticmethod
    def bracket(x):
        return "[%s]" % x

    @staticmethod
    def brace(x):
        return "{%s}" % x

    @staticmethod
    def angle(x):
        return "<%s>" % x

_gcount = 0
class PreppyOutputTestCase(PreppyTestCase):
    def getRunTimeOutput(self, prepCode, quoteFunc=str, lquoteFunc=None, **params):
        "compile code, run with parameters and collect output"

        global _gcount
        mod=preppy.getModule('test_preppy%d'%_gcount,savePyc=0,sourcetext=prepCode)
        _gcount += 1

        collector = []
        mod.run(params, __write__=collector.append, quoteFunc=quoteFunc, lquoteFunc=lquoteFunc)
        output = (quoteFunc or lquoteFunc)('')[0:0].join(collector)
        return output

    def getGetOutput(self,prepCode,*args,**kwds):
        global _gcount
        mod=preppy.getModule('test_preppy%d'%_gcount,savePyc=0,sourcetext=prepCode)
        _gcount += 1
        return mod.get(*args,**kwds)

class GeneratedCodeTestCase(PreppyOutputTestCase):
    """Maybe the simplest and most all-encompassing:
    take a little prep file, compile, exec, and verify that
    output is as expected.  This should catch gross failures
    of preppy """
    def checkStatic(self):
        prepCode = "Hello World"
        out = self.getRunTimeOutput(prepCode)
        self.assertEqual(out, "Hello World")

    def checkExpr1(self):
        prepCode = "Hello {{2+2}} World"
        out = self.getRunTimeOutput(prepCode)
        self.assertEqual(out, "Hello 4 World")

    def checkWhitespaceRespected1(self):
        prepCode = "{{2+2}} " # 1 trailing space
        out = self.getRunTimeOutput(prepCode)
        self.assertEqual(out, "4 ")

    def checkWhitespaceRespected2(self):
        prepCode = "  \t \r{{2+3}}\n " # 1 trailing space
        out = self.getRunTimeOutput(prepCode)
        self.assertEqual(out, "  \t \r5\n ")

    def checkIfStatement1(self):
        prepCode = "Hello, my name is {{name}} and I am a " \
                   "{{if sex=='m'}}guy{{elif sex=='f'}}gal{{else}}neuter{{endif}}."

        out = self.getRunTimeOutput(prepCode, name='fred',sex='m')
        self.assertEqual(out, "Hello, my name is fred and I am a guy.")

        out = self.getRunTimeOutput(prepCode, name='fred',sex='m')
        self.assertEqual(out, "Hello, my name is fred and I am a guy.")

        out = self.getRunTimeOutput(prepCode, name='fred',sex='m')
        self.assertEqual(out, "Hello, my name is fred and I am a guy.")

    def checkFuncWithSideEffects(self):
        prepLines = []
        prepLines.append('{{script}}l = [1, 2]{{endscript}}')
        prepLines.append('{{l.pop()}},')
        prepLines.append('{{l.pop()}},')
        prepLines.append('{{len(l)}}')
        prepCode = ''.join(prepLines)
        out = self.getRunTimeOutput(prepCode)
        self.assertEqual(out, "2,1,0")

    def checkWhileColon(self):
        self.assertEqual(self.getRunTimeOutput('{{script}}i=2{{endscript}}{{while i>=0:}}{{i}}{{if i}},{{endif}}{{script}}i-=1{{endscript}}{{endwhile}}'), "2,1,0")

    def checkWhileNoColon(self):
        self.assertEqual(self.getRunTimeOutput('{{script}}i=2{{endscript}}{{while i>=0}}{{i}}{{if i}},{{endif}}{{script}}i-=1{{endscript}}{{endwhile}}'), "2,1,0")

    def checkForColon(self):
        self.assertEqual(self.getRunTimeOutput('{{for i in (1,2,3):}}{{i}}{{if i!=3}},{{endif}}{{endfor}}'), "1,2,3")

    def checkForNoColon(self):
        self.assertEqual(self.getRunTimeOutput('{{for i in (1,2,3)}}{{i}}{{if i!=3}},{{endif}}{{endfor}}'), "1,2,3")

    def checkIfColon(self):
        self.assertEqual(self.getRunTimeOutput('{{if 1:}}1{{endif}}'), "1")

    def checkIfNoColon(self):
        self.assertEqual(self.getRunTimeOutput('{{if 1}}1{{endif}}'), "1")

    def checkIndentingWithComments(self):
        self.assertEqual(self.getRunTimeOutput('''{{script}}
        #
            i=0
                #
            i=1
    #
        {{endscript}}{{i}}'''),"1")

    def checkForVars(self):
        self.assertEqual(self.getRunTimeOutput('{{i}}{{for i in (1,2,3)}}{{i}}{{if i!=3}},{{endif}}{{endfor}}',i='A'), "A1,2,3")

    def checkLoopVars(self):
        self.assertEqual(self.getRunTimeOutput('{{i}}{{"".join([str(i) for i in (1,2,3)])}}',i='A'), "A123")

    def checkClosingBraces(self):
        self.assertEqual(self.getRunTimeOutput('i}}'), "i}}")

    def checkBackSlashes(self):
        self.assertEqual(self.getRunTimeOutput('{{script}}i=1+2+\\\n\t3{{endscript}}{{i}}'), "6")

    def checkCatchesUnterminated(self):
        self.assertRaises((ValueError,SyntaxError),self.getRunTimeOutput,'{{script}}i=1+2+\\\n\t3{{endscript}}{{i}')

    def checkCatchesIllegalBackSlash(self):
        self.assertRaises((SyntaxError,ValueError),self.getRunTimeOutput,'{{script}}i=1+2+\\ \n\t3{{endscript}}{{i}}')

    def checkQuoting1(self):
        out = self.getRunTimeOutput("<p>{{clientName}}</p>", clientName='Smith & Jones')
        self.assertEqual(out, "<p>Smith & Jones</p>")

    @fposto
    def checkQuoting2(self):
        out = self.getRunTimeOutput("A&B<p>{{clientName}}</p><p>{{script}}print(clientName){{endscript}}</p>", clientName='Smith & Jones')
        self.assertEqual(out, "A&B<p>Smith & Jones</p><p>Smith & Jones\n</p>")

    @fposto
    def checkQuoting3(self):
        def customQuoteFunc(v):
            """Here's a very old, but real, quote function we once used.
            
            Defense against people embedding Javascript in parameters.
            Script kiddies often hope that you will echo through some form
            parameter like a surname in a text field into HTML, so they
            can embed some huge chunk of their own HTMl code in it.
            This ensures any tags in input are escaped and thus deactivated"""

            vs = v.split("&")
            if len(vs)>1:
                # quote ampersands, but not twice
                v = '&'.join([vs[0]]+
                        [((";" not in f or len(f.split(";")[0].split())>1) and 'amp;' or '')+f for f in vs[1:]]
                        )
            return v.replace("<","&lt;").replace(">","&gt;").replace('"',"&quot;").replace("'","&#039;")

        out = self.getRunTimeOutput("A&B<p>{{clientName}}</p><p>{{script}}print(clientName){{endscript}}</p>", clientName='Smith & Jones', quoteFunc=customQuoteFunc)
        self.assertEqual(out, "A&B<p>Smith &amp; Jones</p><p>Smith &amp; Jones\n</p>")

    @fposto
    def checkQuoting4(self):
        def myQuote(x):
            return x.replace("&", "and")
        out = self.getRunTimeOutput("A&B<p>{{clientName}}</p><p>{{script}}print(clientName){{endscript}}</p>", clientName='Smith & Jones', quoteFunc=myQuote)
        self.assertEqual(out, "A&B<p>Smith and Jones</p><p>Smith and Jones\n</p>")

    #preppy's standard quote
    def checkQuoting5(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=1.2, quoteFunc=preppy.uStdQuote), "<a>1.2</a>")

    def checkQuoting6(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A='<&>', quoteFunc=preppy.uStdQuote), "<a>&lt;&amp;&gt;</a>")

    def checkQuoting7(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=preppy.SafeUnicode('<&>'), quoteFunc=preppy.uStdQuote), "<a><&></a>")

    def checkQuoting8(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=preppy.SafeUnicode('<&>'), quoteFunc=preppy.uStdQuote), "<a><&></a>")

    def checkQuoting9(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=b'\xc2\xae', quoteFunc=preppy.uStdQuote), u"<a>\xae</a>")

    def checkQuoting10(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=None, quoteFunc=preppy.uStdQuote), u"<a></a>")

    def checkQuoting11(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=b'A', quoteFunc=preppy.uStdQuote), u"<a>A</a>")

    def checkQuoting12(self):
        self.assertEqual(self.getRunTimeOutput('<a>{{A}}</a>', A=u'A', quoteFunc=preppy.uStdQuote), u"<a>A</a>")

    def checkQuoting13(self):
        self.assertEqual(self.getRunTimeOutput(b'<a>{{A}}</a>', A=b'A', quoteFunc=preppy.bStdQuote), b"<a>A</a>")

    def checkQuoting14(self):
        self.assertEqual(self.getRunTimeOutput(b'<a>{{A}}</a>', A=u'A', quoteFunc=preppy.bStdQuote), b"<a>A</a>")

    def checkSafing1(self):
        self.assertEqual(preppy.SafeString(b'<A>'),b'<A>')

    def checkSafing2(self):
        self.assertEqual(preppy.SafeString(u'<A>'),u'<A>')

    def checkSafing3(self):
        self.assertEqual(preppy.SafeUnicode(b'<A>'),b'<A>')

    def checkSafing4(self):
        self.assertEqual(preppy.SafeUnicode(u'<A>'),u'<A>')

    def checkSafing5(self):
        self.assertEqual(preppy.SafeString(b'<A>').__class__.__name__,'SafeString')

    def checkSafing6(self):
        self.assertEqual(preppy.SafeString(u'<A>').__class__.__name__,'SafeUnicode')

    def checkSafing7(self):
        self.assertEqual(preppy.SafeUnicode(b'<A>').__class__.__name__,'SafeString')

    def checkSafing8(self):
        self.assertEqual(preppy.SafeUnicode(u'<A>').__class__.__name__,'SafeUnicode')

    def checkSafing9(self):
        self.assertEqual(preppy.bStdQuote(preppy.SafeString(b'<A>')),b'<A>')

    def checkSafing10(self):
        self.assertEqual(preppy.bStdQuote(preppy.SafeString(u'<A>')),b'<A>')

    def checkSafing11(self):
        self.assertEqual(preppy.uStdQuote(preppy.SafeUnicode(b'<A>')),u'<A>')

    def checkSafing12(self):
        self.assertEqual(preppy.uStdQuote(preppy.SafeString(u'<A>')),u'<A>')

    fce_src1 = "{{for i in range(3)}}{{if i==C}}{{continue}}{{endif}}{{i}}{{endfor}}"
    fce_src2 = "{{for i in range(3)}}{{if i==C}}{{continue}}{{endif}}{{i}}{{else}}FORELSE{{endfor}}"
    def checkForContinueElse1(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src1, C=-1, quoteFunc=preppy.uStdQuote), "012")

    def checkForContinueElse2(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src1, C=2, quoteFunc=preppy.uStdQuote), "01")

    def checkForContinueElse3(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src1, C=0, quoteFunc=preppy.uStdQuote), "12")
        
    def checkForContinueElse4(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src2, C=-1, quoteFunc=preppy.uStdQuote), "012FORELSE")

    def checkForContinueElse5(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src2, C=2, quoteFunc=preppy.uStdQuote), "01FORELSE")

    def checkForContinueElse6(self):
        self.assertEqual(self.getRunTimeOutput(self.fce_src2, C=0, quoteFunc=preppy.uStdQuote), "12FORELSE")

    fbe_src1="{{for i in range(3)}}{{if i==C}}{{break}}{{endif}}{{i}}{{endfor}}"
    fbe_src2="{{for i in range(3)}}{{if i==C}}{{break}}{{endif}}{{i}}{{else}}FORELSE{{endfor}}"
    def checkForBreakElse1(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src1, C=-1, quoteFunc=preppy.uStdQuote), "012")

    def checkForBreakElse2(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src1, C=2, quoteFunc=preppy.uStdQuote), "01")

    def checkForBreakElse3(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src1, C=0, quoteFunc=preppy.uStdQuote), "")

    def checkForBreakElse4(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src2, C=-1, quoteFunc=preppy.uStdQuote), "012FORELSE")

    def checkForBreakElse5(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src2, C=2, quoteFunc=preppy.uStdQuote), "01")

    def checkForBreakElse6(self):
        self.assertEqual(self.getRunTimeOutput(self.fbe_src2, C=0, quoteFunc=preppy.uStdQuote), "")

    def checkRaisesValueError(self):
        self.assertRaises(ValueError,self.getRunTimeOutput,"{{raise ValueError('aaa')}}")

    def checkBareRaise(self):
        self.assertRaises(ValueError,self.getRunTimeOutput,"{{try}}{{float('aaa')}}{{except}}{{raise}}{{endtry}}")

    def checkCatchesBareReturn(self):
        self.assertRaises(SyntaxError,self.getRunTimeOutput,'{{return}}')

    def checkCatchesBareEnddef(self):
        self.assertRaises(SyntaxError,self.getRunTimeOutput,'{{enddef}}')

    trye_src="""TRY{{try}}{{if i==1}}
RAISE{{raise Exception('zzz')}}{{endif}}
TRYBODY{{except}}
EXCEPT{{endtry}}"""
    def checkTryExcept1(self):
        self.assertEqual(self.getRunTimeOutput(self.trye_src, i=0, quoteFunc=preppy.uStdQuote), "TRY\nTRYBODY")

    def checkTryExcept2(self):
        self.assertEqual(self.getRunTimeOutput(self.trye_src, i=1, quoteFunc=preppy.uStdQuote), "TRY\nRAISE\nEXCEPT")

    tryee_src="""TRY{{try}}{{if i==1}}
raise ValueError{{raise ValueError('bbb')}}{{elif i==2}}
raise TypeError{{raise TypeError('ccc')}}{{elif i==3}}
raise Exception{{raise Exception('zzz')}}{{endif}}{{except ValueError}}
catch ValueError{{except TypeError}}
catch TypeError{{except}}
catch all errors{{else}}
TRYELSE{{finally}}
TRYFINALLY{{endtry}}"""
    def checkTryExceptElseFinally1(self):
        self.assertEqual(self.getRunTimeOutput(self.tryee_src, i=0, quoteFunc=preppy.uStdQuote), "TRY\nTRYELSE\nTRYFINALLY")

    def checkTryExceptElseFinally2(self):
        self.assertEqual(self.getRunTimeOutput(self.tryee_src, i=1, quoteFunc=preppy.uStdQuote), "TRY\nraise ValueError\ncatch ValueError\nTRYFINALLY")

    def checkTryExceptElseFinally3(self):
        self.assertEqual(self.getRunTimeOutput(self.tryee_src, i=2, quoteFunc=preppy.uStdQuote), "TRY\nraise TypeError\ncatch TypeError\nTRYFINALLY")

    def checkTryExceptElseFinally4(self):
        self.assertEqual(self.getRunTimeOutput(self.tryee_src, i=3, quoteFunc=preppy.uStdQuote), "TRY\nraise Exception\ncatch all errors\nTRYFINALLY")

    tryf_src="""TRY{{try}}
FTRY{{try}}{{if i==1}}
raise Exception{{raise Exception('zzz')}}{{endif}}
FTRYBODY{{finally}}
FTRYFINALLY{{endtry}}{{except}}
catch all errors{{endtry}}"""
    def checkTryFinally1(self):
        self.assertEqual(self.getRunTimeOutput(self.tryf_src, i=0, quoteFunc=preppy.uStdQuote), "TRY\nFTRY\nFTRYBODY\nFTRYFINALLY")

    def checkTryFinally1(self):
        self.assertEqual(self.getRunTimeOutput(self.tryf_src, i=1, quoteFunc=preppy.uStdQuote), "TRY\nFTRY\nraise Exception\nFTRYFINALLY\ncatch all errors")

    def checkWith(self):
        fn = preppy.__file__
        self.assertEqual(self.getRunTimeOutput("{{with open(fn,'r') as f}}{{f.name}}{{endwith}}",fn=fn, quoteFunc=preppy.uStdQuote),fn)

    def checkImport1(self):
        self.assertEqual(self.getRunTimeOutput('{{import token}}{{token.__name__}}',quoteFunc=preppy.uStdQuote),'token')

    def checkImport2(self):
        self.assertEqual(self.getRunTimeOutput('{{import token as x}}{{x.__name__}}',quoteFunc=preppy.uStdQuote),'token')

    def checkFrom1(self):
        if isPy310:
            txt = '{{from tkinter import ttk}}{{ttk.__name__}}'
            ans = 'tkinter.ttk'
        else:
            txt = '{{from distutils import cmd}}{{cmd.__name__}}'
            ans = 'distutils.cmd'
        self.assertEqual(self.getRunTimeOutput(txt,quoteFunc=preppy.uStdQuote),ans)

    def checkFrom2(self):
        if isPy310:
            txt = '{{from tkinter import ttk as x}}{{x.__name__}}'
            ans = 'tkinter.ttk'
        else:
            txt = '{{from distutils import cmd as x}}{{x.__name__}}'
            ans = 'distutils.cmd'
        self.assertEqual(self.getRunTimeOutput(txt,quoteFunc=preppy.uStdQuote),ans)

    def checkAssert1(self):
        self.assertRaises(AssertionError,self.getRunTimeOutput,"{{assert i==1}}{{i}}", i=0, quoteFunc=preppy.uStdQuote)

    def checkAssert2(self):
        self.assertEqual(self.getRunTimeOutput("{{assert i==1}}{{i}}", i=1, quoteFunc=preppy.uStdQuote), "1")

    def checkEmptyScript1(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}}{{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyScript2(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}} {{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyScript3(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}}\n{{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyScript4(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}}\n\n{{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyScript5(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}}\n#just a comment{{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyScript6(self):
        self.assertEqual(self.getRunTimeOutput('a{{script}}\n#just a comment\n{{endscript}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyIf1(self):
        self.assertEqual(self.getRunTimeOutput('a{{if False}}{{else}}b{{endif}}c',quoteFunc=preppy.uStdQuote),'abc')

    def checkEmptyIf2(self):
        self.assertEqual(self.getRunTimeOutput('a{{if True}}{{else}}b{{endif}}c',quoteFunc=preppy.uStdQuote),'ac')

    def checkEmptyIfElse1(self):
        self.assertEqual(self.getRunTimeOutput('a{{if False}}b{{else}}{{endif}}c',quoteFunc=preppy.uStdQuote),'ac')

    def checkEmptyIfElse2(self):
        self.assertEqual(self.getRunTimeOutput('a{{if True}}b{{else}}{{endif}}c',quoteFunc=preppy.uStdQuote),'abc')

    def checkEmptyWhile1(self):
        self.assertEqual(self.getRunTimeOutput('a{{while False}}{{endwhile}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyWhile2(self):
        self.assertEqual(self.getRunTimeOutput('{{script}}def I(z=[0,1]):\n return z.pop()\n{{endscript}}a{{while I()}}{{endwhile}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyWhileElse1(self):
        self.assertEqual(self.getRunTimeOutput('a{{while False}}{{else}}{{endwhile}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyWhileElse2(self):
        self.assertEqual(self.getRunTimeOutput('{{script}}def I(z=[0,1]):\n return z.pop()\n{{endscript}}a{{while I()}}{{else}}{{endwhile}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyFor1(self):
        self.assertEqual(self.getRunTimeOutput('a{{for x in ()}}{{endfor}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyFor2(self):
        self.assertEqual(self.getRunTimeOutput('a{{for x in (0,)}}{{endfor}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyForElse1(self):
        self.assertEqual(self.getRunTimeOutput('a{{for x in ()}}{{else}}{{endfor}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyForElse2(self):
        self.assertEqual(self.getRunTimeOutput('a{{for x in (0,)}}{{else}}{{endfor}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyTry(self):
        self.assertEqual(self.getRunTimeOutput('a{{try}}{{except}}{{else}}{{finally}}{{endtry}}b',quoteFunc=preppy.uStdQuote),'ab')

    def checkEmptyWith(self):
        self.assertEqual(self.getRunTimeOutput('a{{with open(fn,'r') as f}}{{endwith}}b',fn=preppy.__file__,quoteFunc=preppy.uStdQuote),'ab')

    if isPy310:
        def checkSimpleMatch(self):
            val = 'Z-404-Z'
            self.assertEqual(self.getRunTimeOutput('''{{for i,status in enumerate((400,404,418,"A"))}}{{i}}({{match status}}{{case 400}}Bad request{{case 404}}Not{{val}}found{{case 418}}I'm a {{'pretty '}}teapot}}{{case _}}Something's wrong with the Internet{{endmatch}}){{endfor}}''', val=val, quoteFunc=preppy.uStdQuote),
                """0(Bad request)1(NotZ-404-Zfound)2(I'm a pretty teapot}})3(Something's wrong with the Internet)""")
            self.assertEqual(self.getRunTimeOutput('''{{for i,status in enumerate((400,404,418,"A"))}}{{i}}({{match status}}    {{case 400}}Bad request{{case 404}}Not{{val}}found{{case 418}}I'm a {{'pretty '}}teapot}}{{case _}}Something's wrong with the Internet{{endmatch}}){{endfor}}''', val=val, quoteFunc=preppy.uStdQuote),
                """0(Bad request)1(NotZ-404-Zfound)2(I'm a pretty teapot}})3(Something's wrong with the Internet)""")
            self.assertRaises(SyntaxError,self.getRunTimeOutput,'''{{for i,status in enumerate((400,404,418,"A"))}}{{i}}({{match status}}   A {{case 400}}Bad request{{case 404}}Not{{val}}found{{case 418}}I'm a {{'pretty '}}teapot}}{{case _}}Something's wrong with the Internet{{endmatch}}){{endfor}}''', val=val, quoteFunc=preppy.uStdQuote)

    def checkLQuotingOld(self):
        '''__lquoteFunc__ applies to the literals'''
        self.assertEqual(
            self.getRunTimeOutput("Hello World",
                lquoteFunc=self.bracket),
                "[Hello World]")

    def checkQuotingOld(self):
        '''__quoteFunc__ applies to the expressions'''
        self.assertEqual(
            self.getRunTimeOutput("{{script}}i=14{{endscript}}Hello World{{i}}",
                quoteFunc=self.brace),
                "Hello World{14}")

    def checkQuotingBothOld(self):
        self.assertEqual(
            self.getRunTimeOutput("{{script}}i=14{{endscript}}Hello World{{i}}",
                quoteFunc=self.brace, lquoteFunc=self.bracket),
                "[Hello World]{14}")

    def checkOverridingStdout(self):
        prepCode = "AAAAA{{script}}print(value){{endscript}}BBBBB"
        oldStdout = sys.stdout
        try:
            #default is now not to override stdout
            sys.stdout = preppy.StringIO()
            self.assertEqual(
                    self.getRunTimeOutput(prepCode,value='\nIIIII'),
                    "AAAAABBBBB")
            self.assertEqual(sys.stdout.getvalue(),'\nIIIII\n')

            sys.stdout = preppy.StringIO()
            self.assertEqual(
                self.getRunTimeOutput(prepCode,value='\nIIIII',__preppyOverrideStdout__=True),
                    "AAAAA\nIIIII\nBBBBB")
            self.assertEqual(sys.stdout.getvalue(),'')

            sys.stdout = preppy.StringIO()
            self.assertEqual(
                self.getRunTimeOutput(prepCode,value='\nIIIII',__preppyOverrideStdout__=False),
                    "AAAAABBBBB")
            self.assertEqual(sys.stdout.getvalue(),'\nIIIII\n')
        finally:
            sys.stdout = oldStdout

    def checkIncludeOverridingStdout(self):
        with open('innercheckIncludeOverridingStdout.prep','wb') as f:
            f.write(b'EEEEE{{script}}print(value){{endscript}}FFFFF')
        outer = "AAAAA{{script}}print(value){{endscript}}{{include('innercheckIncludeOverridingStdout',savePyc=0,value=ivalue)}}BBBBB"
        oldStdout = sys.stdout
        try:
            #default no override
            sys.stdout = preppy.StringIO()
            self.assertEqual(
                self.getRunTimeOutput(outer,value='\nIIIII',ivalue='\nJJJJJ'),
                    "AAAAAEEEEEFFFFFBBBBB")
            self.assertEqual(sys.stdout.getvalue(),'\nIIIII\n\nJJJJJ\n')

            sys.stdout = preppy.StringIO()
            self.assertEqual(
                self.getRunTimeOutput(outer,value='\nIIIII',ivalue='\nJJJJJ',__preppyOverrideStdout__=True),
                    "AAAAA\nIIIII\nEEEEE\nJJJJJ\nFFFFFBBBBB")
            self.assertEqual(sys.stdout.getvalue(),'')

            sys.stdout = preppy.StringIO()
            self.assertEqual(
                self.getRunTimeOutput(outer,value='\nIIIII',ivalue='\nJJJJJ',__preppyOverrideStdout__=False),
                    "AAAAAEEEEEFFFFFBBBBB")
            self.assertEqual(sys.stdout.getvalue(),'\nIIIII\n\nJJJJJ\n')
        finally:
            sys.stdout = oldStdout

    def checkDef(self):
        for c,r in [
                (
                "{{def a(b)}}+++++{{b}}-----{{if b % 2 ==0}}{{return '=%d='%(b*b)}}{{endif}}{{return ''}}{{enddef}}AAAAA{{a(1)}}BBBBB{{a(2)}}CCCCC",
                "AAAAA+++++1-----BBBBB+++++2-----=4=CCCCC"
                ),
                (
                "{{def a(b)}}{{script}}z=7{{endscript}}{{def c(d)}}{{3*d+1}}{{z}}{{enddef}}+++++{{c(b)}}-----{{if b % 2 ==0}}{{return '=%d='%(b*b)}}{{endif}}{{enddef}}AAAAA{{a(1)}}BBBBB{{a(2)}}CCCCC",
                "AAAAA+++++47-----BBBBB+++++77-----=4=CCCCC",
                ),
                ]:
            self.assertEqual(self.getRunTimeOutput(c),r)

    def checkDnlDws(self):
        for c, r in [
                (
                '------ no\x20\x20.dnl/.dws -----\n{{for i in range(3)}}\n\t{{script}}\n\tj = 2**i\n\t{{endscript}}\n\ti={{i}} j={{j}}\n{{endfor}}\n\n--------------------------\n------ with .dnl/.dws -----\n{{for i in range(3)}}{{.dws}}\n\t{{script}}\n\tj = 2**i\n\t{{endscript}}{{.dnl}}\n\ti={{i}} j={{j}}\n{{endfor}}{{.dws}}\n\n--------------------------\n',
                '------ no\x20\x20.dnl/.dws -----\n\n\t\n\ti=0 j=1\n\n\t\n\ti=1 j=2\n\n\t\n\ti=2 j=4\n\n\n--------------------------\n------ with .dnl/.dws -----\n\ti=0 j=1\n\ti=1 j=2\n\ti=2 j=4\n--------------------------\n'
                ),
            ]:
            self.assertEqual(self.getRunTimeOutput(c),r)

class ErrorIndicationTestCase(PreppyOutputTestCase):
    @classmethod
    def setUpClass(cls):
        cls.src0 = ['line %d' % i for i in range(10000)]

    def checkErrorIndication1(self):
        src = '\n'.join(self.src0 + ['{{raise ValueError("AAA")}}'] + self.src0)
        self.assertEqual(checkErrorTextContains('line 10001, in __code__',self.getRunTimeOutput,src,quoteFunc=preppy.uStdQuote),'')

    def checkErrorIndication2(self):
        src = '\n'.join(self.src0 + ['{{raise ValueError("AAA")}}'])
        self.assertEqual(checkErrorTextContains('line 10001, in __code__',self.getRunTimeOutput,src,quoteFunc=preppy.uStdQuote),'')

    def checkErrorIndication3(self):
        src = '\n'.join(['{{raise ValueError("AAA")}}']+self.src0)
        self.assertEqual(checkErrorTextContains('line 1, in __code__',self.getRunTimeOutput,src,quoteFunc=preppy.uStdQuote),'')

    def checkErrorIndication4(self):
        src = '\n'.join(['{{u"\u2019".encode("ascii")}}']+self.src0)
        self.assertEqual(checkErrorTextContains('line 1, in __code__',self.getRunTimeOutput,src,quoteFunc=preppy.uStdQuote),'')

    def checkErrorIndication5(self):
        src = '\n'.join(self.src0 + ['{{u"\u2019".encode("ascii")}}'])
        self.assertEqual(checkErrorTextContains('line 10001, in __code__',self.getRunTimeOutput,src,quoteFunc=preppy.uStdQuote),'')

    def checkErrorIndication6(self):
        src = '\n'.join(self.src0 + ['{{raise ValueError("AAA")}}'] + self.src0)
        self.assertEqual(checkErrorTextContains('line 10001, in get',self.getGetOutput,'{{def()}}'+src),'')

    def checkErrorIndication7(self):
        src = '\n'.join(self.src0 + ['{{raise ValueError("AAA")}}'])
        self.assertEqual(checkErrorTextContains('line 10001, in get',self.getGetOutput,'{{def()}}'+src),'')

    def checkErrorIndication8(self):
        src = '\n'.join(['{{raise ValueError("AAA")}}']+self.src0)
        self.assertEqual(checkErrorTextContains('line 1, in get',self.getGetOutput,'{{def()}}'+src),'')

    def checkErrorIndication9(self):
        src = '\n'.join(['{{u"\u2019".encode("ascii")}}']+self.src0)
        self.assertEqual(checkErrorTextContains('line 1, in get',self.getGetOutput,'{{def()}}'+src),'')

    def checkErrorIndication10(self):
        src = '\n'.join(self.src0 + ['{{u"\u2019".encode("ascii")}}'])
        self.assertEqual(checkErrorTextContains('line 10001, in get',self.getGetOutput,'{{def()}}'+src),'')

    def checkErrorIndication11(self):
        src = '\n'.join(self.src0)
        for line in [
                    '{{if 1}}{{raise ValueError}}{{endif}}',
                    '{{if 0}}{{else}}{{raise ValueError}}{{endif}}',
                    '{{if 0}}{{elif 1}}{{raise ValueError}}{{endif}}',
                    '{{if 0}}{{elif 0}}{{else}}{{raise ValueError}}{{endif}}',
                    '{{while 1}}{{raise ValueError}}{{endwhile}}',
                    '{{while 0}}{{else}}{{raise ValueError}}{{endwhile}}',
                    '{{for i in (0,1)}}{{raise ValueError}}{{endfor}}'
                    '{{for i in (0,1)}}{{else}}{{raise ValueError}}{{endfor}}'
                    '{{script}}raise ValueError{{endscript}}'
                    '{{if 1}}{{script}}raise ValueError{{endscript}}{{endif}}',
                    '{{if 0}}{{else}}{{raise ValueError}}{{endif}}',
                    '{{if 0}}{{elif 1}}{{script}}raise ValueError{{endscript}}{{endif}}',
                    '{{if 0}}{{elif 0}}{{else}}{{script}}raise ValueError{{endscript}}{{endif}}',
                    '{{while 1}}{{script}}raise ValueError{{endscript}}{{endwhile}}',
                    '{{while 0}}{{else}}{{script}}raise ValueError{{endscript}}{{endwhile}}',
                    '{{for i in (0,1)}}{{script}}raise ValueError{{endscript}}{{endfor}}',
                    '{{for i in (0,1)}}{{else}}{{script}}raise ValueError{{endscript}}{{endfor}}',
                    ]:
            self.assertEqual(checkErrorTextContains('line 10001, in __code__',self.getRunTimeOutput,src+'\n'+line+'\n'+src,quoteFunc=preppy.uStdQuote,label=line),'')

class NewGeneratedCodeTestCase(PreppyTestCase):
    """Maybe the simplest and most all-encompassing:
    take a little prep file, compile, exec, and verify that
    output is as expected.  This should catch gross failures
    of preppy """

    def getRunTimeOutput(self, prepCode, *args,**kwds):
        "compile code, run with parameters and collect output"
        mod=preppy.getModule('test_preppy',savePyc=0,sourcetext=prepCode)
        return mod.get(*args,**kwds)

    #def checkSpuriousError(self):
    #    self.assertEqual(2+2, 5)


    def checkNoGet(self):
        self.assertRaises(AttributeError,self.getRunTimeOutput,"Hello World")

    def checkUnicodeName(self):
        mod=preppy.getModule(u'./test_preppy',savePyc=0,sourcetext='Hello World')

    def checkUnicodeDirectory(self):
        mod=preppy.getModule('./test_preppy',directory=u'.',savePyc=0,sourcetext='Hello World')

    def checkUnicodeExtension(self):
        mod=preppy.getModule('./test_preppy',source_extension=u'.prep',savePyc=0,sourcetext='Hello World')

    def checkUnicodeSourceText(self):
        mod=preppy.getModule('./test_preppy',savePyc=0,sourcetext=u'Hello World')

    def checkNoArgs(self):
        self.assertEqual(self.getRunTimeOutput("{{def()}}Hello World"), "Hello World")

    def checkNoArgsNeededA(self):
        self.assertRaises(TypeError,self.getRunTimeOutput,"{{def()}}Hello World",1)
    def checkNoArgsNeededK(self):
        self.assertRaises(TypeError,self.getRunTimeOutput,"{{def()}}Hello World",a=1)

    def checkNoArgsExpr(self):
        self.assertEqual(self.getRunTimeOutput("{{def()}}{{script}}i=5*4{{endscript}}Hello World{{i}}"),
                "Hello World20")

    def checkOneArgA(self):
        self.assertEqual(self.getRunTimeOutput("{{def(a)}}Hello World{{a}}",1), "Hello World1")
    def checkOneArgK(self):
        self.assertEqual(self.getRunTimeOutput("{{def(a)}}Hello World{{a}}",a=1), "Hello World1")

    def checkOneArgNeeded0(self):
        self.assertRaises(TypeError,self.getRunTimeOutput,"{{def(a)}}Hello World{{a}}")
    def checkOneArgNeeded2(self):
        self.assertRaises(TypeError,self.getRunTimeOutput,"{{def(a)}}Hello World{{a}}",1,2)
    def checkOneArgNeededW(self):
        self.assertRaises(TypeError,self.getRunTimeOutput,"{{def(a)}}Hello World{{a}}",b=3)

    def checkBadVar(self):
        self.assertRaises(NameError,self.getRunTimeOutput,"{{def()}}Hello World{{i}}")

    def checkOkVar(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def()}}Hello World{{for i in (1,2)}}{{i}}{{endfor}}"),
            "Hello World12")

    def checkForVar(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def()}}Hello World{{for i in (1,2)}}{{i}}{{endfor}}{{i-2}}"),
            "Hello World120")

    def checkArgs1(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def(a,*args)}}Hello World{{a}}{{if args}}{{args}}{{endif}}",1),
            "Hello World1")
    def checkArgs3(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def(a,*args)}}Hello World{{a}}{{if args}}{{args}}{{endif}}",1,2,3),
            "Hello World1(2, 3)")

    def checkKwds0(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def(a,**kwds)}}Hello World{{a}}{{if kwds}}{{[k for k in sorted(kwds.items())]}}{{endif}}",1),
            "Hello World1")
    def checkKwds2(self):
        self.assertEqual(
            self.getRunTimeOutput(
                "{{def(a,**kwds)}}Hello World{{a}}{{if kwds}}{{[k for k in sorted(kwds.items())]}}{{endif}}",1,b=2,c=3),
            "Hello World1[('b', 2), ('c', 3)]")

    def checkLQuoting(self):
        '''__lquoteFunc__ applies to the literals'''
        self.assertEqual(
            self.getRunTimeOutput("{{def()}}Hello World",
                __lquoteFunc__=self.bracket),
                "[Hello World]")

    def checkQuoting(self):
        '''__quoteFunc__ applies to the expressions'''
        self.assertEqual(
            self.getRunTimeOutput("{{def()}}{{script}}i=14{{endscript}}Hello World{{i}}",
                __quoteFunc__=self.brace),
                "Hello World{14}")

    def checkQuotingBoth(self):
        self.assertEqual(
            self.getRunTimeOutput("{{def()}}{{script}}i=14{{endscript}}Hello World{{i}}",
                __quoteFunc__=self.brace, __lquoteFunc__=self.bracket),
                "[Hello World]{14}")

class OutputModeTestCase(unittest.TestCase):
    """Checks all ways of generating output return identical
    results - grab string, file"""

    def checkOutputModes(self):
        "compile code, run with parameters and collect output"
        prepCode = "Hello, my name is {{name}} and I am a " \
                   "{{if sex=='m'}}guy{{elif sex=='f'}}gal{{else}}neuter{{endif}}."

        params = {'name':'fred','sex':'m'}

        mod=preppy.getModule('test_preppy',savePyc=0,sourcetext=prepCode)

        # this way of making a module should work back to 1.5.2
        # nowadays 'new.module' would be preferable for clarity

        # use write function
        collector = []
        mod.run(params, __write__=collector.append)
        output1 = ''.join(collector)
        from preppy import StringIO
        buf = StringIO()
        mod.run(params, outputfile=buf)
        output2 = buf.getvalue()
        assert output1 == output2, '__write__ and outputfile results differ'

        output3 = mod.getOutput(params)
        assert output3 == output2, 'getOutput(...) and outputfile results differ'

class IncludeTestCase(PreppyTestCase):
    @classmethod
    def setUpClass(cls):
        cls.outer = preppy.getModule('outer',savePyc=0)
    def testRequiredArgs(self):
        def args(*args, **stuff):
            return True
        inner = "{{direction}}"
        modInner = preppy.getModule('dummy1',savePyc=0,sourcetext=inner)
        outer = """{{for direction in directions}}{{inner.getOutput({'direction':direction})}}{{endfor}}"""
        modOuter = preppy.getModule('dummy2',savePyc=0,sourcetext=outer)
        ns = {'directions':('NORTH','SOUTH','EAST','WEST'),
              'inner':modInner}
        output = modOuter.getOutput(ns)
        #self.assertRaises(NameError, modOuter.getOutput, ns)
        self.assertEqual(output, "NORTHSOUTHEASTWEST")

    def testIncludeQuoting(self):
        self.assertEqual(self.outer.getOutput(dict(j='J')),
                'in outer.prep\n\n\tbefore include inner i=0 j=J\n\tin inner.prep v=J*10 w=0\n\n\tbefore include inner1 i=0 j=J\n\tin inner1.prep v=0 w=J*100\n\n\tafter include inner1 i=0 j=J\n\n')
    def testIncludeQuotingQ(self):
        self.assertEqual(self.outer.getOutput(dict(j='J'),quoteFunc=self.bracket),
                'in outer.prep\n\n\tbefore include inner i=[0] j=[J]\n\t[in inner.prep v=[J*10] w=[0]\n]\n\tbefore include inner1 i=[0] j=[J]\n\t[in inner1.prep v=[0] w=[J*100]\n]\n\tafter include inner1 i=[0] j=[J]\n\n'
                )
    def testIncludeQuotingL(self):
        self.assertEqual(self.outer.getOutput(dict(j='J'),lquoteFunc=self.angle),
                '<in outer.prep\n><\n\tbefore include inner i=>0< j=>J<\n\t><in inner.prep v=>J*10< w=>0<\n><\n\tbefore include inner1 i=>0< j=>J<\n\t><in inner1.prep v=>0< w=>J*100<\n><\n\tafter include inner1 i=>0< j=>J<\n><\n>'
                )
    def testIncludeQuotingQL(self):
        self.assertEqual(self.outer.getOutput(dict(j='J'),lquoteFunc=self.angle,quoteFunc=self.bracket),
                '<in outer.prep\n><\n\tbefore include inner i=>[0]< j=>[J]<\n\t>[<in inner.prep v=>[J*10]< w=>[0]<\n>]<\n\tbefore include inner1 i=>[0]< j=>[J]<\n\t>[<in inner1.prep v=>[0]< w=>[J*100]<\n>]<\n\tafter include inner1 i=>[0]< j=>[J]<\n><\n>'
                )

def makeSuite():
    return unittest.TestSuite((
                mkSuite(NewGeneratedCodeTestCase,'check'),
                mkSuite(GeneratedCodeTestCase,'check'),
                mkSuite(OutputModeTestCase,'check'),
                mkSuite(IncludeTestCase),
                mkSuite(ErrorIndicationTestCase,'check'),
                ))

if __name__=='__main__':
    runner = unittest.TextTestRunner()
    runner.run(makeSuite())
